using System;
using System.IO;
using System.Runtime.InteropServices;

/// <summary>
/// Veobecn asova s vysokm rozlenm
/// </summary>
public class HighResTimer
{
	#region API Deklarcia
    [System.Security.SuppressUnmanagedCodeSecurity] 
    [DllImport("kernel32")]
    private static extern bool QueryPerformanceFrequency(ref long PerformanceFrequency);
    [System.Security.SuppressUnmanagedCodeSecurity] 
    [DllImport("kernel32")]
    private static extern bool QueryPerformanceCounter(ref long PerformanceCount);
    #endregion

    #region Deklarcia
    private long numberTicksPerSec = 0;
    private long startTime = 0;
    private long uplynulyCas = 0;
    private long lastTime = 0;

    private bool isTimerRunning = false;
	#endregion

    #region Metdy
    /// <summary>
    /// Inicializcia asovaa
    /// </ Summary>
    public HighResTimer()
    {
        // Pouijeme QueryPerformanceFrequency () na zskanie frekvencie asovaa
        long ticksPerSec = 0;
        if (QueryPerformanceFrequency(ref ticksPerSec))
        {
            numberTicksPerSec = ticksPerSec;
        }
        else
        {
       // Ak nememe poui QueryPerformanceFrequency () vyhodme vnimku
            throw new InvalidOperationException("V systm nepodporuje asova s vysokm rozlienm.");
        }
    }

    /// <summary>
    /// tartuje asova
    /// </summary>
    public void Start()
    {
        if (!QueryPerformanceCounter(ref startTime))
        {
            throw new InvalidOperationException("Chyba pri zskavan poslednej hodnoty asovaa");
        }
        lastTime = startTime;
        isTimerRunning = true;
    }
     /// <summary>
 /// Zastavenie asovaa
 /// </ Summary>
    public void Stop()
    {
        Aktualizuj();
        isTimerRunning = false;
    }

    /// <summary>
    /// Volan raz za snmok
    /// </summary>
    public void Aktualizuj()
    {
        if (!isTimerRunning)
            return;
        // Ulome posledn as
        uplynulyCas = lastTime;

        if (!QueryPerformanceCounter(ref lastTime))
        {
            throw new InvalidOperationException("Chyba pri zskavan poslednej hodnoty asovaa");
        }
    }

    #endregion

    #region Vlastnosti
    /// <summary>
    /// Vracia true ak asova be, inak false
    /// </summary>
    public bool IsRunning
    {
        get { return isTimerRunning; }
    }
    /// <summary>
    /// Poet seknd od poslednej aktualizcie
    /// </summary>
    public float ElapsedTime
    {
        get
        {
            return ((float)(lastTime - uplynulyCas) / (float)numberTicksPerSec);
        }
    }
    /// <summary>
    /// Poet seknd od spustenia asovaa
    /// </summary>
    public float TotalTime
    {
        get
        {
            return ((float)(lastTime - startTime) / (float)numberTicksPerSec);
        }
    }
    #endregion
}
