using System;
using System.IO;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Obrancovia
{
    /// <summary>
    /// Hlavn objekt rovne naej hry
    /// </summary>
    public class Uroven
    {
        #region Konstanty
        private const float Rozostup = 3.35f;
        private const int VelkostPlochy = 13;
        private const float StartovnaPozicia = (Rozostup * VelkostPlochy) / 2.0f;
        #endregion

        #region Premenne triedy
        // Zoznam kociek, ktor tvoria rove.
        private Kocka[] kocky = null;
        // Cieov farba pre vhru.
        private FarbaKocky cielovaFarba;
        // Ulome hern postavu.
        private Player hernaPostava = null;
        // Index kocky, na ktorej stoj hern postava.
        private int indexKockyPostavy;
        // Aktulny poet ahov.
        private int celkomTahov = 0;
        // Maximlny povolen poet ahov.
        private int maximalneTahov = 0;

        // Maximlny povolen as.
        private float maximalnyCas = 0.0f;
        // as, ktor uplynul.
        private float uplynulyCas = 0.0f;

        // as, ktor zostv.
        private TimeSpan zostavajuciCas;

        // M b hra z nejakho dvodu ukonen?
        private bool jeKoniecHry = false;
        // Vyhral hr hru?
        private bool jeHraVyhrana = false;
        // Je to prv ukonenie hry?
        private bool jeKoniecHryPrvy = true;

        // Celkov poet kociek.
        private int pocetKociek = 0;
        // Poet kociek spravnej farby.
        private int pocetSpravnychKociek = 0;
        #endregion

        #region Konstruktor
        public unsafe Uroven(int uroven, Device zariadenie, Player postava)
        {
            // Skontroluje parameter.
            if (postava == null)
            {
                throw new ArgumentNullException("postava");
            }

            // Ulo objekt hernej postavy.
            hernaPostava = postava;
            // Nejprv mus nata sbor rovne. Pri otevrn souboru bude
            // vyhozena vjimka, pokud soubor neexistuje.
            using (FileStream suborUrovne = File.OpenRead(HerneJadro.MediaPath +
              string.Format("Uroven{0}.lvl", uroven)))
            {

                byte[] dataMaximalnyCas = new byte[sizeof(float)];
                suborUrovne.Read(dataMaximalnyCas, 0, dataMaximalnyCas.Length);
                maximalnyCas = BitConverter.ToSingle(dataMaximalnyCas, 0);

                byte[] dataMaximalneTahov = new byte[sizeof(int)];
                suborUrovne.Read(dataMaximalneTahov, 0, dataMaximalneTahov.Length);
                maximalneTahov = BitConverter.ToInt32(dataMaximalneTahov, 0);

                byte[] dataPocetFarieb = new byte[sizeof(int)];
                suborUrovne.Read(dataPocetFarieb, 0, dataPocetFarieb.Length);
                int pocetFarieb = BitConverter.ToInt32(dataPocetFarieb, 0);

                FarbaKocky[] farby = new FarbaKocky[pocetFarieb];
                for (int i = 0; i < barvy.Length; i++)
                {
                    farby[i] = (FarbaKocky)suborUrovne.ReadByte();
                }

                cielovaFarba = (FarbaKocky)suborUrovne.ReadByte();

                byte[] dataPretekajiFarby = new byte[sizeof(bool)];
                souborUrovne.Read(dataPretekajiBarvy, 0, dataPretekajiFarby.Length);
                bool pretekajiFarby = BitConverter.ToBoolean(dataPretekajiFarby, 0);

                kocky = new Kocka[VelkostPlochy * VelkostPlochy];

                int indexKocky = 0;

                for (int j = 0; j < VelkostPlochy; j++)
                {
                    for (int i = 0; i < VelkostPlochy; i++)
                    {
                        byte docasnaFarbaKocky = (byte)suborUrovne.ReadByte();

                        if (docasnaFarbaKocky != 0xff)
                        {
                            Kocka b = new Kocka(zariadenie, farby, docasnaFarbaKocky,
                              pretekajiFarby);
                            b.Umiestnenie = new Vector3(-StartovnaPozicia +
                              (Rozostup * i), -1.5f,
                              -StartovnaPozicia + (Rozostup * j));

                            kocky[indexKocky++] = b;
                        }
                        else
                        {
                            indexKocky++;
                        }
                    }
                }

                byte[] dataIndexKockyPostavy = new byte[sizeof(int)];
                suborUrovne.Read(dataIndexKockyPostavy, 0,
                  dataIndexKockyPostavy.Length);
                indexKockyPostavy = BitConverter.ToInt32(dataIndexKockyPostavy, 0);
            }
            AktualizujPostavuAKocku(true, indexKockyPostavy);

        }
        #endregion

        #region Ovladanie hernej postavy
        public void OnKeyPress(System.Windows.Forms.Keys klavesa)
        {
            switch (klavesa)
            {
                case System.Windows.Forms.Keys.Right:
                    PosunPostavuX(true);
                    break;
                case System.Windows.Forms.Keys.Left:
                    PosunPostavuX(false);
                    break;
                case System.Windows.Forms.Keys.Up:
                    PosunPostavuZ(false);
                    break;
                case System.Windows.Forms.Keys.Down:
                    PosunPostavuZ(true);
                    break;
            }
        }

        private bool AktualizujPostavuAKocku(bool okamzite,
            int novyIndexKockyPostavy)
        {
            Kocka b = kocky[novyIndexKockyPostavy] as Kocka;

            if (okamzite)
            {
                hernaPostava.Umiestnenie = new Vector3(b.Umiestnenie.X, 0, b.Umiestnenie.Z);
            }
            else
            {
                if (!hernaPostava.PosunNa(new Vector3(b.Umiestnenie.X, 0, b.Umiestnenie.Z)))
                {
                    return false;
                }
            }

            b.VyberDalsiuFarbu();
            return true;
        }

        /// <param name="doprava">Pohybuje sa postava doprava?</param>
        /// <returns>true, ak se me postava pohn, inak false</returns>
        private bool MozeSaPostavaPohnutHorizontalne(bool doprava)
        {
            if (((indexKockyPostavy % VelkostPlochy) == 0) && (!doprava))
            {
                return false;
            }
            else if (((indexKockyPostavy % VelkostPlochy) ==
              (VelkostPlochy - 1)) && (doprava))
            {
                return false;
            }
            return (kocky[indexKockyPostavy + ((doprava) ? 1 : -1)] != null);
        }
        /// <param name="kladny">Pohybuje sa postava v kladnom smere z?</param>
        /// <returns>true, ak se me pohn, inak false</returns>
        private bool MozeSaPostavaPohnutVertikalne(bool kladny)
        {
            if (((indexKockyPostavy / VelkostPlochy) == 0) && (kladny))
            {
                return false;
            }
            else if (((indexKockyPostavy / VelkostPlochy) ==
             (VelkostPlochy - 1)) && (!kladny))
            {
                return false;
            }
            return (kocky[indexKockyPostavy + ((kladny) ? -VelkostPlochy :
              VelkostPlochy)] != null);
        }
        /// <param name="kladny">Pohybuje sa postava v kladnom smere z?</param>
        /// <returns>true, ak sa posunula, inak false</returns>
        public bool PosunPostavuZ(bool kladny)
        {
            if (jeKoniecHry)
                return false;
            hernaPostava.NastavSmer(kladny ? SmerPostavy.Dolu :
              SmerPostavy.Hore);
            if (MozeSaPostavaPohnutVertikalne(kladny))
            {
                if (AktualizujPostavuAKocku(false, indexKockyPostavy + ((kladny) ?
                  -VelkostPlochy : VelkostPlochy)))
                {
                    indexKockyPostavy += (kladny) ? -VelkostPlochy : VelkostPlochy;
                    celkomTahov++;
                    return true;
                }
            }
            return false;
        }
        /// <param name="doprava">Pohybuje sa postava doprava?</param>
        /// <returns>true, ak sa posunula, inak false</returns>
        public bool PosunPostavuX(bool doprava)
        {
            if (jeKoniecHry)
                return false;
            hernaPostava.NastavSmer(doprava ? SmerPostavy.Doprava :
              SmerPostavy.Dolava);
            if (MozeSaPostavaPohnutHorizontalne(doprava))
            {
                if (AktualizujPostavuAKocku(false, indexKockyPostavy + ((doprava) ?
                  1 : -1)))
                {
                    indexKockyPostavy += (doprava) ? 1 : -1;
                    celkomTahov++;
                    return true;
                }
            }

            return false;
        }
        #endregion

        #region Aktualizacie
        public void Aktualizuj(float cas)
        {
            uplynulyCas += cas;
            if (maximalnyCas > uplynulyCas)
            {
                zostavajuciCas = new TimeSpan(0, 0, (int)(maximalnyCas - uplynulyCas));
            }
            if (HraVyhrana(uplynulyCas))
            {
                jeKoniecHry = true;
                jeHraVyhrana = true;
            }
            else if ((uplynulyCas >= maximalnyCas) || (celkomTahov > maximalneTahov))
            {
                jeKoniecHry = true;
                jeHraVyhrana = false;
            }
            if ((jeKoniecHry) && (!jeHraVyhrana))
            {
                if (jeKoniecHryPrvy)
                {
                    jeKonecHryPrvy = false;
                    Random r = new Random();
                    int index = 0;
                    foreach (Kocka b in kocky)
                    {
                        if ((b != null) && (indexKockyPostavy != index))
                        {
                            float x, y, z = 0.0f;
                            x = (float)r.NextDouble() * Kocka.MaximumVelocity;
                            y = (float)r.NextDouble() * (Kocka.MaximumVelocity * 2);
                            z = (float)r.NextDouble() * Kocka.MaximumVelocity;
                            if (r.Next(50) > 25) { x *= -1; }
                            if (r.Next(50) > 25) { y *= -1; }
                            if (r.Next(50) > 25) { z *= -1; }
                            b.NastavRychlostKocky(new Vector3(x, y, z));
                        }
                        index++;
                    }
                }
                foreach (Kocka b in kocky)
                {
                    if (b != null)
                    {
                        b.AktualizujKocku(cas);
                    }
                }
            }
        }

        private bool HraVyhrana(float celkovyCas)
        {
            bool vyhral = true;
            pocetKociek = 0;
            pocetSpravnychKociek = 0;
            foreach (Kocka b in kocky)
            {
                if (b != null)
                {
                    pocetKociek++;
                    b.NastavCasKocky(celkovyCas);
                    if (b.FarbaKocky != cielovaFarba)
                    {
                        b.NastavBlikanieKocky(true);
                        vyhral = false;
                    }
                    else
                    {
                        b.NastavBlikanieKocky(false);
                        pocetSpravnychKociek++;
                    }
                }
            }

            return vyhral;
        }

        /// <param name="uroven">Poradie urovne</param>
        /// <returns>true, ak subor existuje; false inak</returns>
        public static bool SkontrolujExistenciuSuboru(int uroven)
        {
            return File.Exists(HerneJadro.MediaPath +
              string.Format("Uroven{0}.lvl", uroven));
        }
        #endregion

        #region Vlastnosti
        public int PocetKociek
        {
            get { return pocetKociek; }
        }
        public int PocetSpravnychKociek
        {
            get { return pocetSpravnychKociek; }
        }
        public string ZostavajuciCas
        {
            get { return zostavajuciCas.ToString(); }
        }
        public int CelkomTahov
        {
            get { return celkemTahov; }
        }
        public int MaximalneTahov
        {
            get { return maximalneTahov; }
        }
        public bool JeKoniecHry
        {
            get { return jeKoniecHry; }
        }
        public bool JeHraVyhrana
        {
            get { return jeHraVyhrana; }
        }
        #endregion

        #region Zobrazenie
        /// <param name="zarizeni">Zobrazovacie zariadenie.</param>
        public void Draw(Device zariadenie)
        {
            foreach (Kocka b in kocky)
            {
                if (b != null)
                {
                    b.Draw(zarizeni);
                }
            }
        }
        #endregion

    }
}